﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation

Imports System.Windows.Media.Media3D
Imports System.Windows.Media.Imaging

Partial Public Class Window1
	Public Sub New()
		MyBase.New()

		Me.InitializeComponent()

		' Insert code required on object creation below this point.
	End Sub

    ' Build the model.
    Private Sub Window_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles MyBase.Loaded
        cboLevel.SelectedIndex = m_Level
        BuildModel()
    End Sub

    ' Make the gasket at the new level.
    Private Sub cboLevel_SelectionChanged(ByVal sender As System.Object, ByVal e As System.Windows.Controls.SelectionChangedEventArgs) Handles cboLevel.SelectionChanged
        m_Level = cboLevel.SelectedIndex
        BuildModel()
    End Sub

#Region "Model Building"
    Private m_Level As Integer = 1

    ' Build the model.
    Private Sub BuildModel()
        If Me.GasketModel Is Nothing Then Exit Sub

        ' Start from scratch.
        For i As Integer = Me.GasketModel.Children.Count - 1 To 0 Step -1
            If TypeOf Me.GasketModel.Children(i) Is GeometryModel3D Then
                Me.GasketModel.Children.RemoveAt(i)
            End If
        Next i

        ' Make the material.
        Dim image_source As New BitmapImage(New Uri("wood.jpg", UriKind.Relative))
        Dim br As New ImageBrush(image_source)
        br.ViewportUnits = BrushMappingMode.Absolute
        Dim the_material As New DiffuseMaterial(br)

        ' Make the gasket.
        MakeGasket(Me.GasketModel, the_material, m_Level, _
            -1, -1, -1, _
            1, 1, 1, _
            0, 0, 0, _
            0.75, 0.75, 0.75)
    End Sub

    ' Make a gasket in the indicated cube.
    Private Sub MakeGasket(ByVal model_group As Model3DGroup, ByVal model_material As Material, _
      ByVal level As Integer, _
      ByVal xmin As Single, ByVal ymin As Single, ByVal zmin As Single, ByVal xmax As Single, ByVal ymax As Single, ByVal zmax As Single, _
      ByVal umin As Single, ByVal vmin As Single, ByVal wmin As Single, ByVal umax As Single, ByVal vmax As Single, ByVal wmax As Single)
        If level > 0 Then
            ' Recurse.
            level -= 1
            Dim x1 As Single = xmin + (xmax - xmin) / 3
            Dim x2 As Single = xmax - (xmax - xmin) / 3
            Dim y1 As Single = ymin + (ymax - ymin) / 3
            Dim y2 As Single = ymax - (ymax - ymin) / 3
            Dim z1 As Single = zmin + (zmax - zmin) / 3
            Dim z2 As Single = zmax - (zmax - zmin) / 3
            Dim u1 As Single = umin + (umax - umin) / 3
            Dim u2 As Single = umax - (umax - umin) / 3
            Dim v1 As Single = vmin + (vmax - vmin) / 3
            Dim v2 As Single = vmax - (vmax - vmin) / 3
            Dim w1 As Single = wmin + (wmax - wmin) / 3
            Dim w2 As Single = wmax - (wmax - wmin) / 3
            ' Front.
            ' Front bottom.
            MakeGasket(model_group, model_material, level, xmin, ymin, zmin, x1, y1, z1, umin, vmin, wmin, u1, v1, w1)
            MakeGasket(model_group, model_material, level, x1, ymin, zmin, x2, y1, z1, u1, vmin, wmin, u2, v1, w1)
            MakeGasket(model_group, model_material, level, x2, ymin, zmin, xmax, y1, z1, u2, vmin, wmin, umax, v1, w1)

            ' Front middle.
            MakeGasket(model_group, model_material, level, xmin, y1, zmin, x1, y2, z1, umin, v1, wmin, u1, v2, w1)
            MakeGasket(model_group, model_material, level, x2, y1, zmin, xmax, y2, z1, u2, v1, wmin, umax, v2, w1)

            ' Front top.
            MakeGasket(model_group, model_material, level, xmin, y2, zmin, x1, ymax, z1, umin, v2, wmin, u1, vmax, w1)
            MakeGasket(model_group, model_material, level, x1, y2, zmin, x2, ymax, z1, u1, v2, wmin, u2, vmax, w1)
            MakeGasket(model_group, model_material, level, x2, y2, zmin, xmax, ymax, z1, u2, v2, wmin, umax, vmax, w1)

            ' Middle.
            ' Middle bottom.
            MakeGasket(model_group, model_material, level, xmin, ymin, z1, x1, y1, z2, umin, vmin, w1, u1, v1, w2)
            MakeGasket(model_group, model_material, level, x2, ymin, z1, xmax, y1, z2, u2, vmin, w1, umax, v1, w2)

            ' Middle top.
            MakeGasket(model_group, model_material, level, xmin, y2, z1, x1, ymax, z2, umin, v2, w1, u1, vmax, w2)
            MakeGasket(model_group, model_material, level, x2, y2, z1, xmax, ymax, z2, u2, v2, w1, umax, vmax, w2)

            ' Back.
            ' Back bottom.
            MakeGasket(model_group, model_material, level, xmin, ymin, z2, x1, y1, zmax, umin, vmin, w2, u1, v1, wmax)
            MakeGasket(model_group, model_material, level, x1, ymin, z2, x2, y1, zmax, u1, vmin, w2, u2, v1, wmax)
            MakeGasket(model_group, model_material, level, x2, ymin, z2, xmax, y1, zmax, u2, vmin, w2, umax, v1, wmax)

            ' Back middle.
            MakeGasket(model_group, model_material, level, xmin, y1, z2, x1, y2, zmax, umin, v1, w2, u1, v2, wmax)
            MakeGasket(model_group, model_material, level, x2, y1, z2, xmax, y2, zmax, u2, v1, w2, umax, v2, wmax)

            ' Back top.
            MakeGasket(model_group, model_material, level, xmin, y2, z2, x1, ymax, zmax, umin, v2, w2, u1, vmax, wmax)
            MakeGasket(model_group, model_material, level, x1, y2, z2, x2, ymax, zmax, u1, v2, w2, u2, vmax, wmax)
            MakeGasket(model_group, model_material, level, x2, y2, z2, xmax, ymax, zmax, u2, v2, w2, umax, vmax, wmax)
        Else
            ' Draw a box here.
            MakeBox(model_group, model_material, _
                xmin, ymin, zmin, xmax, ymax, zmax, _
                umin, vmin, wmin, umax, vmax, wmax)
        End If
    End Sub

    ' Make a box.
    Private Sub MakeBox(ByVal model_group As Model3DGroup, ByVal model_material As Material, _
      ByVal xmin As Single, ByVal ymin As Single, ByVal zmin As Single, ByVal xmax As Single, ByVal ymax As Single, ByVal zmax As Single, _
      ByVal umin As Single, ByVal vmin As Single, ByVal wmin As Single, ByVal umax As Single, ByVal vmax As Single, ByVal wmax As Single)
        ' Top.
        MakeRectangle(model_group, model_material, _
            New Point3D(xmin, ymax, zmin), _
            New Point3D(xmin, ymax, zmax), _
            New Point3D(xmax, ymax, zmax), _
            New Point3D(xmax, ymax, zmin), _
            New Point(umin, wmin), _
            New Point(umin, wmax), _
            New Point(umax, wmax), _
            New Point(umax, wmin))
        ' Bottom.
        MakeRectangle(model_group, model_material, _
            New Point3D(xmax, ymin, zmin), _
            New Point3D(xmax, ymin, zmax), _
            New Point3D(xmin, ymin, zmax), _
            New Point3D(xmin, ymin, zmin), _
            New Point(umax, wmin), _
            New Point(umax, wmax), _
            New Point(umin, wmax), _
            New Point(umin, wmin))
        ' Front.
        MakeRectangle(model_group, model_material, _
            New Point3D(xmin, ymin, zmin), _
            New Point3D(xmin, ymax, zmin), _
            New Point3D(xmax, ymax, zmin), _
            New Point3D(xmax, ymin, zmin), _
            New Point(umin, vmin), _
            New Point(umin, vmax), _
            New Point(umax, vmax), _
            New Point(umax, vmin))
        ' Back.
        MakeRectangle(model_group, model_material, _
            New Point3D(xmax, ymin, zmax), _
            New Point3D(xmax, ymax, zmax), _
            New Point3D(xmin, ymax, zmax), _
            New Point3D(xmin, ymin, zmax), _
            New Point(umax, vmin), _
            New Point(umax, vmax), _
            New Point(umin, vmax), _
            New Point(umin, vmin))
        ' Left.
        MakeRectangle(model_group, model_material, _
            New Point3D(xmin, ymin, zmax), _
            New Point3D(xmin, ymax, zmax), _
            New Point3D(xmin, ymax, zmin), _
            New Point3D(xmin, ymin, zmin), _
            New Point(wmax, vmin), _
            New Point(wmax, vmax), _
            New Point(wmin, vmax), _
            New Point(wmin, vmin))
        ' Right.
        MakeRectangle(model_group, model_material, _
            New Point3D(xmax, ymin, zmin), _
            New Point3D(xmax, ymax, zmin), _
            New Point3D(xmax, ymax, zmax), _
            New Point3D(xmax, ymin, zmax), _
            New Point(wmin, vmin), _
            New Point(wmin, vmax), _
            New Point(wmax, vmax), _
            New Point(wmax, vmin))
    End Sub

    ' Make an outwardly-oriented square with texture.
    Private Sub MakeRectangle(ByVal model_group As Model3DGroup, ByVal model_material As Material, _
        ByVal pt0 As Point3D, ByVal pt1 As Point3D, ByVal pt2 As Point3D, ByVal pt3 As Point3D, _
        ByVal u0 As Point, ByVal u1 As Point, ByVal u2 As Point, ByVal u3 As Point)

        ' Define the new mesh geometry.
        Dim new_mesh As New MeshGeometry3D
        ' Define the corner points.
        Dim pt3Ds() As Point3D = { _
            New Point3D(pt0.X, pt0.Y, pt0.Z), _
            New Point3D(pt1.X, pt1.Y, pt1.Z), _
            New Point3D(pt2.X, pt2.Y, pt2.Z), _
            New Point3D(pt3.X, pt3.Y, pt3.Z) _
        }
        new_mesh.Positions = New Point3DCollection(pt3ds)

        ' Define which points make up the triangles.
        new_mesh.TriangleIndices = New Int32Collection()
        new_mesh.TriangleIndices.Add(0)
        new_mesh.TriangleIndices.Add(1)
        new_mesh.TriangleIndices.Add(2)

        new_mesh.TriangleIndices.Add(0)
        new_mesh.TriangleIndices.Add(2)
        new_mesh.TriangleIndices.Add(3)

        ' Set the texture coordinates.
        new_mesh.TextureCoordinates = New PointCollection()
        new_mesh.TextureCoordinates.Add(u0)
        new_mesh.TextureCoordinates.Add(u1)
        new_mesh.TextureCoordinates.Add(u2)
        new_mesh.TextureCoordinates.Add(u3)

        ' Set the model's material.
        Dim new_model As New GeometryModel3D(new_mesh, model_material)

        ' Add the new model to the model group.
        model_group.Children.Add(new_model)
    End Sub
#End Region ' Model Building

End Class
